import pytest
import moto
import boto3
from awssert.fixture import awssert


@pytest.fixture
def mock_s3():
    with moto.mock_s3():
        yield


def test_s3_bucket_contains_object_assertion(mock_s3, awssert):
    bucket = boto3.resource("s3").Bucket("mock")
    bucket.create(CreateBucketConfiguration={"LocationConstraint": "eu-west-1"})
    bucket.put_object(Key="foo", Body=b"123")
    assert bucket.should.contain("foo")
    assert bucket.does.contain("foo")
    assert bucket.should_not.contain("bar")
    assert bucket.does_not.contain("bar")


def test_s3_bucket_empty_assertion(mock_s3, awssert):
    bucket = boto3.resource("s3").Bucket("mock")
    bucket.create(CreateBucketConfiguration={"LocationConstraint": "eu-west-1"})
    assert bucket.should_be.empty()
    bucket.put_object(Key="foo", Body=b"123")
    assert bucket.should_not_be.empty()


def test_s3_bucket_exists_assertion(mock_s3, awssert):
    bucket = boto3.resource("s3").Bucket("mock")
    assert bucket.should_not.exist()
    assert bucket.does_not.exist()
    bucket.create(CreateBucketConfiguration={"LocationConstraint": "eu-west-1"})
    assert bucket.should.exist()
    assert bucket.does.exist()



from awssert.prefixes import prefixes, AssertionPrefixes
from awssert.core import AssertionPrefixRouter, BotoObjectProxyRegister, BotoObjectProxy


class TableAssertions:
    @prefixes(["should_be", "should_not_be"])
    def empty(self, table):
        return table.item_count == 0

    @prefixes(["should_have", "has", "should_not_have", "does_not_have"])
    def item(self, table, key):
        return "Item" in table.get_item(Key=key)

    @prefixes(["should_have", "has", "should_not_have", "does_not_have"])
    def key(self, table, key):
        keys = [key["AttributeName"] for key in table.key_schema]
        return key in keys


def register_dynamodb_assertions(class_attributes, base_classes, **kwargs):
    proxy = BotoObjectProxy()
    base_classes.insert(0, BotoObjectProxyRegister)
    class_attributes["proxy"] = proxy
    for prefix in AssertionPrefixes.all:
        class_attributes[prefix] = AssertionPrefixRouter(
            prefix, TableAssertions(), proxy
        )



from awssert.exceptions import PrefixNotSupportedError


class AssertionPrefixes:

    _register = {}
    positive = ["should", "does", "should_be", "should_have", "has"]
    negative = [
        "should_not",
        "does_not",
        "does_not_have",
        "should_not_be",
        "should_not_have",
    ]
    all = positive + negative

    @classmethod
    def register_method(cls, method, allowed_prefixes):
        class_of_method = method.__qualname__.split(".")[0]
        if class_of_method not in cls._register:
            cls._register_class(class_of_method)
        for prefix in allowed_prefixes:
            cls._register_class_method(class_of_method, prefix, method)

    @classmethod
    def get_methods_allowing_prefix(cls, on_class, prefix):
        return cls._register[on_class.__name__][prefix]

    @classmethod
    def _register_class(cls, class_name):
        cls._register[class_name] = {prefix: {} for prefix in cls.all}

    @classmethod
    def _register_class_method(cls, class_name, keyword, method):
        try:
            cls._register[class_name][keyword][method.__name__] = method
        except KeyError:
            raise PrefixNotSupportedError(f"Keyword '{keyword}' is not supported")


def prefixes(allowed_prefixes):
    def registrar(method):
        AssertionPrefixes.register_method(method, allowed_prefixes)
        return method

    return registrar



import pytest
import boto3

from awssert.s3 import register_s3_assertions
from awssert.dynamodb import register_dynamodb_assertions


@pytest.fixture
def awssert():
    default_session = boto3._get_default_session()
    default_session.events.register(
        "creating-resource-class.s3.Bucket", register_s3_assertions
    )
    default_session.events.register(
        "creating-resource-class.dynamodb.Table", register_dynamodb_assertions
    )



import functools

from awssert.prefixes import AssertionPrefixes
from awssert.exceptions import DisallowedPrefixOnMethodError


class BotoObjectProxyRegister(object):
    def __init__(self, *args, **kwargs):
        super(BotoObjectProxyRegister, self).__init__(*args, **kwargs)
        self.proxy.reference = self


class BotoObjectProxy:
    def __init__(self):
        self.reference = None


class AssertionPrefixRouter:
    def __init__(self, prefix, assertions_class, proxy):
        self.prefix = prefix
        self.route_to = assertions_class
        self.proxy = proxy
        self.routable_methods = self._setup_routable_methods()

    def _setup_routable_methods(self):
        routable_methods = AssertionPrefixes.get_methods_allowing_prefix(
            self.route_to.__class__, self.prefix
        )
        all_methods = [
            method
            for method in dir(self.route_to)
            if callable(getattr(self.route_to, method)) and not method.startswith("__")
        ]
        for method in all_methods:
            self.__setattr__(method, functools.partial(self._route, method))
        return routable_methods.keys()

    def _route(self, method, *args, **kwargs):
        if method in self.routable_methods:
            result = getattr(self.route_to, method)(
                self.proxy.reference, *args, **kwargs
            )
            return result if self.prefix in AssertionPrefixes.positive else not result
        else:
            raise DisallowedPrefixOnMethodError(
                f"Method '{method}' cannot be used with prefix '{self.prefix}'"
            )



import pytest
import moto
import boto3
from awssert.fixture import awssert


@pytest.fixture
def mock_ddb():
    with moto.mock_dynamodb2():
        yield


def test_table_empty_assertion(mock_ddb, awssert):
    boto3.client("dynamodb").create_table(
        AttributeDefinitions=[{"AttributeName": "mock", "AttributeType": "S"}],
        TableName="mock",
        KeySchema=[{"AttributeName": "mock", "KeyType": "HASH"}],
    )
    table = boto3.resource("dynamodb").Table("mock")
    assert table.should_be.empty()
    table.put_item(Item={"mock": "foo"})
    assert table.should_not_be.empty()


def test_table_has_item_assertion(mock_ddb, awssert):
    boto3.client("dynamodb").create_table(
        AttributeDefinitions=[{"AttributeName": "mock", "AttributeType": "S"}],
        TableName="mock",
        KeySchema=[{"AttributeName": "mock", "KeyType": "HASH"}],
    )
    table = boto3.resource("dynamodb").Table("mock")
    assert table.should_not_have.item({"mock": "foo"})
    assert table.does_not_have.item({"mock": "foo"})
    assert table.does_not_have.item({"mock": "bar"})
    table.put_item(Item={"mock": "foo"})
    assert table.should_have.item({"mock": "foo"})
    assert table.has.item({"mock": "foo"})
    assert table.does_not_have.item({"mock": "bar"})


def test_table_has_key_assertion(mock_ddb, awssert):
    boto3.client("dynamodb").create_table(
        AttributeDefinitions=[{"AttributeName": "mock", "AttributeType": "S"}],
        TableName="mock",
        KeySchema=[{"AttributeName": "mock", "KeyType": "HASH"}],
    )
    table = boto3.resource("dynamodb").Table("mock")
    assert table.should_have.key("mock")
    assert table.has.key("mock")
    assert table.should_not_have.key("foo")
    assert table.does_not_have.key("foo")



from setuptools import setup, find_packages
from pathlib import Path

version = "0.0.3"
here = Path(__file__).parent.resolve()

with open(here.joinpath("README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="awssert",
    version=version,
    description="Declarative assertions for AWS",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/TSNoble/awssert",
    author="Tom Noble",
    author_email="t.s.noble@outlook.com",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Framework :: AWS CDK",
        "Framework :: Pytest",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    keywords=["aws", "python", "pytest", "boto3", "testing"],
    packages=find_packages(),
    python_requires=">=3.5",
    install_requires=[
        "pytest",
        "boto3",
    ],
    project_urls={
        "Bug Reports": "https://github.com/TSNoble/awssert/issues",
        "Source": "https://github.com/TSNoble/awssert",
    }
)



from awssert.prefixes import prefixes, AssertionPrefixes
from awssert.core import AssertionPrefixRouter, BotoObjectProxyRegister, BotoObjectProxy


class BucketAssertions:
    @prefixes(["should", "should_not", "does", "does_not"])
    def contain(self, bucket, key):
        objects = bucket.objects.filter(Prefix=key)
        return any([key == object.key for object in objects])

    @prefixes(["should_be", "should_not_be"])
    def empty(self, bucket):
        return len(list(bucket.objects.all())) == 0

    @prefixes(["should", "should_not", "does", "does_not"])
    def exist(self, bucket):
        return bucket.creation_date is not None


def register_s3_assertions(class_attributes, base_classes, **kwargs):
    proxy = BotoObjectProxy()
    base_classes.insert(0, BotoObjectProxyRegister)
    class_attributes["proxy"] = proxy
    for prefix in AssertionPrefixes.all:
        class_attributes[prefix] = AssertionPrefixRouter(
            prefix, BucketAssertions(), proxy
        )
